/*
 * TranslationUnitStore.h
 *
 *  Created on: 10.01.2013
 *      Author: kaeshammer
 */

#ifndef EXTRACT_LCFRS_TRANSLATIONUNITSTORE_H_
#define EXTRACT_LCFRS_TRANSLATIONUNITSTORE_H_

#include <set>
#include <map>

#include "Alignment.h"

namespace LCFRS {

typedef std::map<int, int> IntMap;

class TranslationUnit {
public:
  TranslationUnit();
  ~TranslationUnit();
  void addToSource(int i);
  void addToTarget(int i);
  const std::set<int>& getSourceIndeces() const;
  const std::set<int>& getTargetIndeces() const;
  std::string toString() const;
  std::string toStringTU() const;
  std::string toStringTUWithoutUnaligned(IntMap& in2outSource, IntMap& in2outTarget) const;
  bool operator<(const TranslationUnit& other) const;
  bool isOnlySource() const {
    return m_targetInd.size() == 0;
  }
  bool isOnlyTarget() const {
    return m_sourceInd.size() == 0;
  }
private:
  std::set<int> m_sourceInd;
  std::set<int> m_targetInd;
};

class TranslationUnitStore {
private:
  std::vector<TranslationUnit> m_tus;
  std::map<int, TranslationUnit> m_sourceTus;
  std::map<int, TranslationUnit> m_targetTus;
  void addToTU(Moses::Alignment& alignment, TranslationUnit& tu,
      int i, std::vector<bool>& sourceTreated, std::vector<bool>& targetTreated);

public:
  unsigned int numSource;
  unsigned int numTarget;
  TranslationUnitStore(Moses::Alignment& alignment,
      std::vector<std::string> sourceTokens, std::vector<std::string> targetTokens);
  std::string toString() const;
  std::string toStringTU() const;
  std::string toStringTUWithoutUnaligned(IntMap& in2outSource, IntMap& in2outTarget) const;
  const std::vector<TranslationUnit>& getTUs() const {
    return m_tus;
  };
  virtual ~TranslationUnitStore(){};
};

std::ostream& operator<<(std::ostream& out, const TranslationUnitStore& r);


}

#endif /* EXTRACT_LCFRS_TRANSLATIONUNITSTORE_H_ */
